#include "../headers/qtexedit.h"
#include <QDebug>

Completor::Completor(QWidget *parent):
    QListWidget(parent)
{
    theEdit=NULL;
}

void Completor::keyPressEvent ( QKeyEvent * e ){
    if(e->key() == Qt::Key_Enter ||e->key() == Qt::Key_Return){
        //qDebug()<<"{Completor::keyPressEven()}Enter is pressed."<<endl;
        QListWidgetItem* it = this->currentItem();
        emit(submit(it));
        return;
    }else if(e->key() == Qt::Key_Space){
        //qDebug()<<"{Completor::keyPressEven()}Space is pressed."<<endl;
        QListWidgetItem* it = this->currentItem();
        emit(submit(it));
    }else if(e->key() == Qt::Key_Down){
        if(currentRow() >= count()-1){
            setCurrentRow(0);
            return;
        }
    }else if(e->key() == Qt::Key_Up){
        if(currentRow() == 0){
            setCurrentRow(count()-1);
            return;
        }
    }else if(e->key() == Qt::Key_K || e->key() == Qt::Key_W){
        if(currentRow() == 0){
            setCurrentRow(count()-1);
            return;
        }else{
            setCurrentRow(currentRow()-1);
            return;
        }
    }else if(e->key() == Qt::Key_J || e->key() == Qt::Key_S){
        if(currentRow() >= count()-1){
            setCurrentRow(0);
            return;
        }else{
            setCurrentRow(currentRow()+1);
            return;
        }
    }else if(e->key() == Qt::Key_Tab){
        if(theEdit!=NULL){
            theEdit->setFocus();
        }
        this->clear();
        this->hide();
        return;
    }
    QListWidget::keyPressEvent(e);
}
void Completor::keyReleaseEvent ( QKeyEvent * e ){
    QListWidget::keyReleaseEvent(e);
}


QTeXEdit::QTeXEdit(QWidget *parent) :
    QPlainTextEdit(parent)
{
    cmp = NULL;
    sChecker = NULL;    
}

void QTeXEdit::applySpellDict(QString dicName, QString path){
    if(sChecker!=NULL){
        delete sChecker;
        sChecker = NULL;
    }
    QString affpath= path+dicName+".aff";
    QString dicpath= path+dicName+".dic";
    sChecker = new Hunspell(affpath.toLatin1(),dicpath.toLatin1());
}

void QTeXEdit::setCompleter(Completor* w){
    cmp = w;
    w->theEdit = this;
    //connect(cmp,SIGNAL(itemActivated(QListWidgetItem*)),this,SLOT(activateItem(QListWidgetItem*)));
    connect(cmp, SIGNAL(submit(QListWidgetItem*)),this,SLOT(activateItem(QListWidgetItem*)));
}

#include <QMenu>
void QTeXEdit::contextMenuEvent(QContextMenuEvent *event)
{
    QMenu *menu = createStandardContextMenu();
    //if(!textCursor().selectedText().isEmpty()){
    //    menu->addAction(tr("Check grammar/expression..."),this,SLOT());
    //}
    //...
    menu->exec(event->globalPos());
    delete menu;
}


void QTeXEdit::keyPressEvent ( QKeyEvent * e ){
    if(cmp!=NULL && (! cmp->isHidden())){
        cmp->setCurrentRow(0);
        if(e->key()==Qt::Key_Up
                || (e->key()== Qt::Key_K && (e->modifiers()==Qt::ControlModifier||e->modifiers()==Qt::MetaModifier))
                ){
                cmp->setFocus();
                cmp->setCurrentRow(cmp->count()-1);
                return;
        }else if(e->key()==Qt::Key_Down
                 || (e->key()== Qt::Key_J && (e->modifiers()==Qt::ControlModifier||e->modifiers()==Qt::MetaModifier))
                     ||e->key()== Qt::Key_Tab
                 ){
                cmp->setFocus();
                int idx = 1;
                if(cmp->count()==1){idx=0;}
                cmp->setCurrentRow(idx);
                return;
        }else if(e->key()==Qt::Key_Left || e->key()== Qt::Key_Home || e->key()== Qt::Key_End ||
                 e->key()== Qt::Key_PageDown || e->key()== Qt::Key_PageUp ||e->key() == Qt::Key_Escape
                 ){
            cmp->clear();
            cmp->hide();
            this->setFocus();
            return;
        }else if(e->key()==Qt::Key_Right){
            QString txt = cmp->currentItem()->text();
            emit(askToComplete(txt));
            this->setFocus();
            cmp->clear();
            cmp->hide();
            this->setFocus();
            return;
        }
        else if(e->key()  == Qt::Key_Space || e->key() == Qt::Key_Enter || e->key() == Qt::Key_Return){
            QString txt = cmp->currentItem()->text();
            emit(askToComplete(txt));
            this->setFocus();
            return;
        }else if(e->key() == Qt::Key_Escape){
            cmp->clear();
            cmp->hide();
            this->setFocus();
            return;
        }
    }

    if(e->matches(QKeySequence::Copy)){
        //qDebug()<<"QKeySequence::Copy pressd"<<endl;
        copy();
        return;
    }
    else if(e->matches(QKeySequence::Cut)){
        //qDebug()<<"QKeySequence::Cut pressd"<<endl;
        cut();
        return;
    }

    else if(e->key()==Qt::Key_Comma && (e->modifiers()==Qt::ControlModifier||e->modifiers()==Qt::MetaModifier)){
        //emit(askToGoogleSearch());
        return;
    }
    else if(e->key()==Qt::Key_Equal && (e->modifiers()==Qt::ControlModifier||e->modifiers()==Qt::MetaModifier)){
        return;
    }else if(e->key()==Qt::Key_Slash && (e->modifiers()==Qt::ControlModifier||e->modifiers()==Qt::MetaModifier)){
        return;
    }

    QPlainTextEdit::keyPressEvent(e);
}

void QTeXEdit::keyReleaseEvent ( QKeyEvent * e ){
    if(e->matches(QKeySequence::Copy)){
        this->copy();
        return;
    }
    else if(e->matches(QKeySequence::Cut)){
        this->cut();
        return;
    }

    else if(e->key()==Qt::Key_M && (e->modifiers()==Qt::ControlModifier||e->modifiers()==Qt::MetaModifier)){
        QString word ="";
        if(textCursor().hasSelection()){
            word = textCursor().selectedText();
            emit(askToDict(word));
        }else{
            QTextCursor cur=textCursor();
            cur.movePosition(QTextCursor::StartOfWord);
            cur.movePosition(QTextCursor::EndOfWord,QTextCursor::KeepAnchor);//.select(QTextCursor::WordUnderCursor);
            setTextCursor(cur);
                word = cur.selectedText();
                //qDebug()<<"Word ="<<word<<"++"<<endl;

                for(int tt =0;tt<3;tt++){
                if(word =="" ||word == " " || word == "," || word == ";" || word == "." ||word == "-"){
                    cur=textCursor();
                    cur.movePosition(QTextCursor::PreviousWord);  //qDebug()<<"<--"<<endl;
                    cur.movePosition(QTextCursor::PreviousWord);  //qDebug()<<"<--"<<endl;
                    cur.movePosition(QTextCursor::StartOfWord); //qDebug()<<"|<-"<<endl;
                    cur.movePosition(QTextCursor::EndOfWord,QTextCursor::KeepAnchor);  //qDebug()<<"|-->|"<<endl;
                    setTextCursor(cur);
                    word = cur.selectedText();
                    //qDebug()<<"Word ="<<word<<"++"<<endl;
                }else{
                    break;
                }}

                if(word =="" ||word == " " || word == "," || word == ";" || word == "." ||word == "-"){
                    word ="";
                }
                cur.movePosition(QTextCursor::EndOfWord);
                setTextCursor(cur);
            emit(askToDict(word));
        }
        return;
    }else if(e->key()==Qt::Key_Equal && (e->modifiers()==Qt::ControlModifier||e->modifiers()==Qt::MetaModifier)){
        QString word ="";
        if(textCursor().hasSelection()){
            word = textCursor().selectedText();
            emit(askToWiki(word));
        }else{
            QTextCursor cur=textCursor();
            cur.movePosition(QTextCursor::StartOfWord);
            cur.movePosition(QTextCursor::EndOfWord,QTextCursor::KeepAnchor);//.select(QTextCursor::WordUnderCursor);
            setTextCursor(cur);
                word = cur.selectedText();
                //qDebug()<<"Word ="<<word<<"++"<<endl;

                for(int tt =0;tt<3;tt++){
                if(word =="" ||word == " " || word == "," || word == ";" || word == "." ||word == "-"){
                    cur=textCursor();
                    cur.movePosition(QTextCursor::PreviousWord);  //qDebug()<<"<--"<<endl;
                    cur.movePosition(QTextCursor::PreviousWord);  //qDebug()<<"<--"<<endl;
                    cur.movePosition(QTextCursor::StartOfWord); //qDebug()<<"|<-"<<endl;
                    cur.movePosition(QTextCursor::EndOfWord,QTextCursor::KeepAnchor);  //qDebug()<<"|-->|"<<endl;
                    setTextCursor(cur);
                    word = cur.selectedText();
                    //qDebug()<<"Word ="<<word<<"++"<<endl;
                }else{
                    break;
                }}

                if(word =="" ||word == " " || word == "," || word == ";" || word == "." ||word == "-"){
                    word ="";
                }
                cur.movePosition(QTextCursor::EndOfWord);
                setTextCursor(cur);
            emit(askToWiki(word));
        }
        return;
    }
    else if(e->key()==Qt::Key_Comma && (e->modifiers()==Qt::ControlModifier||e->modifiers()==Qt::MetaModifier)){
        emit(askToGoogleSearch());
        return;
    }else if(e->key()==Qt::Key_Slash && (e->modifiers()==Qt::ControlModifier||e->modifiers()==Qt::MetaModifier)){
        QString word ="";
        if(textCursor().hasSelection()){
            word = textCursor().selectedText();
        }
        emit(askToGinger(word));
        //emit(askToGinger(QString));
        return;
    }


    QPlainTextEdit::keyReleaseEvent(e);
}

void QTeXEdit::activateItem(QListWidgetItem* it){
    QString txt = it->text();
    emit(askToComplete(txt));
}

#include <QApplication>
#include <QClipboard>

void QTeXEdit::copy(){
    //qDebug()<<"My Copying...";
    QString txt = this->textCursor().selectedText();
    QString nl = QString::fromUtf8("\u2029");
    txt = txt.replace(nl,"\n");
    qApp->clipboard()->setText(txt);
    return;
}
void QTeXEdit::cut(){
    //qDebug()<<"My Cutting...";
    QString txt = this->textCursor().selectedText();
    QString nl = QString::fromUtf8("\u2029");
    txt = txt.replace(nl,"\n");

    QPlainTextEdit::cut();

    qApp->clipboard()->setText(txt);
    return;
}


void QTeXEdit::inputMethodEvent ( QInputMethodEvent * e ){
    //qDebug()<<"Commiting String: {"<<e->commitString()<<"}"<<endl;
    QPlainTextEdit::inputMethodEvent(e);
}

void QTeXEdit::mousePressEvent(QMouseEvent *e){
    if(cmp!=NULL && (! cmp->isHidden())){
        cmp->clear();
        cmp->hide();
        this->setFocus();
    }
    QPlainTextEdit::mousePressEvent(e);
}
